package mdd.client;

import javafx.geometry.Point2D;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.Tab;
import javafx.scene.input.*;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

public class WorkBench extends Tab implements IGUIEventClient{
    private ScrollPane _scrollPane;
    public Processor _processor;
    private Point2D _pos = new Point2D(0.0f,0.0f);
    private GUIEventHandler _eventHandler;

    public WorkBench(Processor processor){
        super(processor.Name);

        _eventHandler = GUIEventHandler.getEventHandler();
        _eventHandler.addEventListener(this);
        _processor = processor;
        setId(_processor.getAsPrefix().toString());
        _scrollPane = new ScrollPane(_processor.all);
        setContent(_scrollPane);
        //setContent(_processor.all);

        _scrollPane.addEventHandler(DragEvent.DRAG_DROPPED, e -> {
            Dragboard db = e.getDragboard();
            boolean success = false;
            if (db.hasString()) {
                JSONParser parser = new JSONParser();
                JSONObject json = null;
                try {
                    json = (JSONObject)parser.parse(db.getString());
                } catch (ParseException parseException) {
                    parseException.printStackTrace();
                }
                if (json.containsKey("server")){
                    if(json.get("server").equals(Client.client.getID())){
                        if(json.containsKey("add")){
                            //System.out.println("[WorkBench]: Drag detected: " + json.toString());
                            JSONObject jmsg = new JSONObject();
                            jmsg.put("operation", "add");
                            JSONObject jargs = new JSONObject();
                            jargs.put("subject",  _processor.getJSONIdentifier());
                            jargs.put("object",   json.get("add"));
                            jmsg.put("args", jargs);
                            _eventHandler.publishEvent(10,"try", jmsg);
                            success = true;
                        }
                    }
                }
            }
            e.setDropCompleted(success);
            e.consume();
        });

        _scrollPane.addEventHandler(MouseEvent.MOUSE_PRESSED, e -> {
            if(e.getButton() == MouseButton.PRIMARY) {
                JSONObject json = new JSONObject();
                json.put("type", "optimizer");
                _eventHandler.publishEvent("MOUSE_PRESSED", json);
            }
            if(e.getButton() == MouseButton.SECONDARY) {

            }
            e.consume();
        });
    }

    @Override
    public void processGUIEvent(String event, JSONObject args) {

    }

    @Override
    public void close() throws Exception {
        _eventHandler.removeEventListener(this);
    }
}
